/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cz.xelfi.quoridor.desktop.sample;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.api.client.WebResource;
import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameId;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.core.MediaType;
import org.openide.util.Lookup;

/** Class that connects to the server and answers queries.
 *
 * @author Jaroslav Tulach
 */
abstract class Quoridor {
    private static Quoridor q;

    public static Quoridor getDefault() {
        if (q != null) {
            return q;
        }
        q = Lookup.getDefault().lookup(Quoridor.class);
        if (q == null) {
            q = new Impl();
        }
        return q;
    }

    public abstract List<GameId> listGames();
    public abstract Game getGame(String id);

    public abstract boolean login(String login, String password);

    private static class Impl extends Quoridor {
        private final WebResource wr;
        private String id = "";
        private static final Logger LOG = Logger.getLogger(Impl.class.getName());

        public Impl() {
            Client c = new Client();
            wr = c.resource("http://quoridor.xelfi.cz/api/");
        }

        @Override
        public List<GameId> listGames() {
            GenericType<List<GameId>> gType = new GenericType<List<GameId>>() {};
            List<GameId> ids = wr.path("games").queryParam("loginID", id).
                accept(MediaType.TEXT_XML).get(gType);
            return ids;
        }


        @Override
        public Game getGame(String id) {
            Game g = wr.path("games").path(id).queryParam("loginID", id).
                accept(MediaType.TEXT_XML).get(Game.class);
            return g;
        }

        @Override
        public boolean login(String login, String password) {
            try {
                id = wr.path("login").queryParam("name", login).
                    queryParam("password", password).put(String.class);
                return id != null;
            } catch (UniformInterfaceException ex) {
                LOG.log(Level.INFO, "Cannot login", ex);
                return false;
            }
        }

    }
}
