/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.emailer

import scala.xml._
import java.net.URL
import java.net.HttpURLConnection
import java.io.BufferedReader
import java.io.InputStreamReader
import scala.io._
import scala.collection.Map
import scala.collection.mutable.HashMap
import javax.mail.internet._
import javax.mail.Message
import javax.mail.Transport

object Main {
  def main(args: Array[String]) {
    if (args == null || args.size != 4) {
      Console.println("Usage: emailer.jar <quoridor API URL> <login name> <password> <smtp server IP>")
      return
    }

    val api = new URL(args(0));

    val loginID = login(api, args(1), args(2))

    val okDelay = 3600 * 1000 * 24
    val props = java.lang.System.getProperties
    props.put("mail.smtp.host", args(3));

    val session = if (args(3).equals("print")) null else javax.mail.Session.getDefaultInstance(props, null)

    val notify = new Notify() {
      protected def allPlayers(): Map[String,List[Info]] = {
        val u = new URL(api, "games");
        val conn = u.openConnection
        conn.setRequestProperty("Accept", "text/xml")

        val ret = new HashMap[String,List[Info]]

        val before = System.currentTimeMillis - okDelay
        val games = XML.load(conn.getInputStream)
        for (val g : Node <- games \\ "gameId") {
          val modified = (g \\ "@modified").text.toLong
          if (modified < before) {
            val status = (g \\ "@status").text;
            val name =
              if (status == "blackMove") {
                (g \\ "@black").text
              } else if (status == "whiteMove") {
                (g \\ "@white").text
              } else {
                null
              }
            if (name != null) {
              val prev = ret.getOrElse(name, Nil)
              val id = (g \\ "@id").text
              val info = new Info(modified, id, name)
              val next = info :: prev
              ret.put(name, next);
            }
          }
        }
        return ret;
      }

      protected def emails(ids : Collection[String]): Map[String,String] = {
        var ret = new HashMap[String,String]
        for (val p <- ids) {
          val u = new URL(api, "users/" + p + "?loginID=" + loginID)
          val conn = u.openConnection
          conn.setRequestProperty("Accept", "text/xml")
          val xml = XML.load(conn.getInputStream)
          val res = (xml \\ "property").filter({
              n => n.attribute("name") match {
                case Some(attr) => attr.text == "email"
                case _ => false
              }
          })
          if (!res.isEmpty) {
            ret.put(p, res(0).text)
          }
        }
        return ret
      }

      protected def sendEmail(address : String, subject : String, text : String) : Unit = {
        Console.println("Sending message to")
        Console.println("  email: " + address.split('@')(0) + "@... ")
        Console.println("  subject: " + subject)
        Console.println("  " + text)
        if (session == null) {
          return;
        }

        val message = new MimeMessage(session)
        message.setFrom(new InternetAddress("quoridor@xelfi.cz"))
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(address))
        message.setSubject(subject)
        message.setText(text)

        Transport.send(message)
      }

      protected def resignGame(id : Info): Unit = {
        val u = new URL(api, "games/" + id.gameId + "?loginID=" + loginID + "&player=" + id.toMove + "&move=RESIGN")
        val c = u.openConnection.asInstanceOf[HttpURLConnection]
        c.setRequestMethod("PUT")
        c.connect
        val is = c.getInputStream
        val r = new BufferedReader(new InputStreamReader(is))
        var goOn = true;
        while (goOn) {
          val s = r.readLine()
          if (s == null) {
            goOn = false;
          } else {
            Console.println(s);
          }
        }
      }
    }

    notify.process()
  }

  def login(url : URL, name : String, password : String) : String = {
    val u = new URL(url, "login?name=" + name + "&password=" + password);
    val conn = u.openConnection.asInstanceOf[java.net.HttpURLConnection]
    conn.setRequestMethod("PUT")
    conn.setDoInput(true)
    conn.connect
    val reply = Source.fromInputStream(conn.getInputStream)
    return reply.mkString("")
  }
}
