/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cz.xelfi.quoridor.visidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Move;
import java.awt.GridLayout;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.swing.JPanel;

/** JavaBean class that displays and can manipulate with Quoridor {@link Board}.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public final class BoardPane extends JPanel {
    private final Viewer viewer;
    private final List<BoardListener> listeners = new CopyOnWriteArrayList<BoardListener>();
    
    public BoardPane() {
        setLayout(new GridLayout(1,1));
        viewer = new Viewer();
        viewer.moveListener(this);
        add(viewer);
    }
    
    public final void setBoard(Board b) {
        viewer.setBoard(b);
    }
    
    public final Board getBoard() {
        return viewer.getBoard();
    }

    /** The board can either be in <q>view only</q> mode when showing state 
     * of the {@link Board} during opponent's move, or it can be editable.
     * In editable mode, one shall listen on changes delivered to 
     * BoardListener.
     * 
     * @param editable read only or editable mode?
     */
    public final void setEditable(boolean editable) {
        viewer.enableListeners(editable);
    }
    
    public final void addBoardListener(BoardListener l) {
        listeners.add(l);
    }
    public final void removeBoardListener(BoardListener l) {
        listeners.remove(l);
    }
    
    final void moveHappened(Move m) {
        BoardEvent ev = null;
        for (BoardListener bl : listeners) {
            if (ev == null) {
                ev = new BoardEvent(this, m);
            }
            bl.boardChanged(ev);
        }
    }
}
